module Utils

"Converts a vector of vectors into a matrix. The inner vectors do not need to have the same length.

All entries that cannot be filled have ```invalidElement``` as their value
"
function convert_to_matrix(vec::Vector{Vector{T}}, invalidElement::T)::Matrix{T} where T
	vecCols = get_max_inner_length(vec)
	vecRows = length(vec)
	vecMat = fill(invalidElement, vecCols, vecRows)
	
	for i in eachindex(vec)
		vecMat[:,i] = copyto!(vecMat[:,i], vec[i])
	end

	return vecMat
end

"Retrieves the number of entries for the largest inner vector"
function get_max_inner_length(vec::Vector{Vector{T}})::Int where T
	maxLength = 0
	@inbounds for i in eachindex(vec)
		if length(vec[i]) > maxLength
			maxLength = length(vec[i])
		end
	end

	return maxLength
end

"Returns a CuArray filed with the data provided. The inner vectors do not have to have the same length. All missing elements will be the value ```invalidElement```"
function create_cuda_array(data::Vector{Vector{T}}, invalidElement::T)::CuArray{T} where T
	dataCols = Utils.get_max_inner_length(data)
	dataRows = length(data)
	dataMat = Utils.convert_to_matrix(data, invalidElement)
	cudaArr = CuArray{T}(undef, dataCols, dataRows) # length(parameters) == number of expressions
	copyto!(cudaArr, dataMat)

	return cudaArr
end

end